@@ -25,7 +25,6 @@
 #define PCI_CHIP_MACH32 0x4158
 #define PCI_CHIP_RS250_4237 0x4237
 #define PCI_CHIP_R200_BB 0x4242
-#define PCI_CHIP_R200_BC 0x4243
 #define PCI_CHIP_RS100_4336 0x4336
 #define PCI_CHIP_RS200_4337 0x4337
 #define PCI_CHIP_MACH64CT 0x4354
@@ -458,6 +457,10 @@
 #define PCI_CHIP_RS880_9713 0x9713
 #define PCI_CHIP_RS880_9714 0x9714
 #define PCI_CHIP_RS880_9715 0x9715
+#define PCI_CHIP_PALM_9802 0x9802
+#define PCI_CHIP_PALM_9803 0x9803
+#define PCI_CHIP_PALM_9804 0x9804
+#define PCI_CHIP_PALM_9805 0x9805
 #define PCI_CHIP_CYPRESS_6880 0x6880
 #define PCI_CHIP_CYPRESS_6888 0x6888
 #define PCI_CHIP_CYPRESS_6889 0x6889
@@ -497,3 +500,53 @@
 #define PCI_CHIP_CEDAR_68F8 0x68F8
 #define PCI_CHIP_CEDAR_68F9 0x68F9
 #define PCI_CHIP_CEDAR_68FE 0x68FE
+#define PCI_CHIP_CAYMAN_6700 0x6700
+#define PCI_CHIP_CAYMAN_6701 0x6701
+#define PCI_CHIP_CAYMAN_6702 0x6702
+#define PCI_CHIP_CAYMAN_6703 0x6703
+#define PCI_CHIP_CAYMAN_6704 0x6704
+#define PCI_CHIP_CAYMAN_6705 0x6705
+#define PCI_CHIP_CAYMAN_6706 0x6706
+#define PCI_CHIP_CAYMAN_6707 0x6707
+#define PCI_CHIP_CAYMAN_6708 0x6708
+#define PCI_CHIP_CAYMAN_6709 0x6709
+#define PCI_CHIP_CAYMAN_6718 0x6718
+#define PCI_CHIP_CAYMAN_6719 0x6719
+#define PCI_CHIP_CAYMAN_671C 0x671C
+#define PCI_CHIP_CAYMAN_671D 0x671D
+#define PCI_CHIP_BARTS_6720 0x6720
+#define PCI_CHIP_BARTS_6721 0x6721
+#define PCI_CHIP_BARTS_6722 0x6722
+#define PCI_CHIP_BARTS_6723 0x6723
+#define PCI_CHIP_BARTS_6724 0x6724
+#define PCI_CHIP_BARTS_6725 0x6725
+#define PCI_CHIP_BARTS_6726 0x6726
+#define PCI_CHIP_BARTS_6727 0x6727
+#define PCI_CHIP_BARTS_6728 0x6728
+#define PCI_CHIP_BARTS_6729 0x6729
+#define PCI_CHIP_BARTS_6738 0x6738
+#define PCI_CHIP_BARTS_6739 0x6739
+#define PCI_CHIP_TURKS_6740 0x6740
+#define PCI_CHIP_TURKS_6741 0x6741
+#define PCI_CHIP_TURKS_6742 0x6742
+#define PCI_CHIP_TURKS_6743 0x6743
+#define PCI_CHIP_TURKS_6744 0x6744
+#define PCI_CHIP_TURKS_6745 0x6745
+#define PCI_CHIP_TURKS_6746 0x6746
+#define PCI_CHIP_TURKS_6747 0x6747
+#define PCI_CHIP_TURKS_6748 0x6748
+#define PCI_CHIP_TURKS_6749 0x6749
+#define PCI_CHIP_TURKS_6750 0x6750
+#define PCI_CHIP_TURKS_6758 0x6758
+#define PCI_CHIP_TURKS_6759 0x6759
+#define PCI_CHIP_CAICOS_6760 0x6760
+#define PCI_CHIP_CAICOS_6761 0x6761
+#define PCI_CHIP_CAICOS_6762 0x6762
+#define PCI_CHIP_CAICOS_6763 0x6763
+#define PCI_CHIP_CAICOS_6764 0x6764
+#define PCI_CHIP_CAICOS_6765 0x6765
+#define PCI_CHIP_CAICOS_6766 0x6766
+#define PCI_CHIP_CAICOS_6767 0x6767
+#define PCI_CHIP_CAICOS_6768 0x6768
+#define PCI_CHIP_CAICOS_6770 0x6770
+#define PCI_CHIP_CAICOS_6779 0x6779
